from typing import overload
from urllib.parse import ParseResult, SplitResult

class URL:
    def __init__(self, url: str | ParseResult | SplitResult) -> None: ...
    def __bool__(self) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    @overload
    @classmethod
    def objectify(cls, url: None) -> None: ...
    @overload
    @classmethod
    def objectify(cls, url: URL | str | ParseResult | SplitResult) -> URL: ...
    def __getattr__(self, attr: str): ...
    def __unicode__(self) -> str: ...
    def strip_trailing_slash(self) -> URL: ...
    def is_auth(self) -> bool: ...
    def unauth(self) -> URL: ...
    def canonical(self) -> URL: ...
    def join(self, path: object) -> URL: ...

@overload
def make(url: None) -> None: ...
@overload
def make(url: URL | str | ParseResult | SplitResult) -> URL: ...



from _typeshed import Self
from collections.abc import Iterable
from typing import Any, ClassVar
from typing_extensions import TypeAlias

_Element: TypeAlias = Any  # actually lxml.etree._Element

class BaseElement:
    tag: ClassVar[str | None]
    children: list[BaseElement]
    value: str | None
    attributes: Any | None
    caldav_class: Any | None
    def __init__(self, name: str | None = ..., value: str | bytes | None = ...) -> None: ...
    def __add__(self: Self, other: BaseElement) -> Self: ...
    def xmlelement(self) -> _Element: ...
    def xmlchildren(self, root: _Element) -> None: ...
    def append(self: Self, element: BaseElement | Iterable[BaseElement]) -> Self: ...

class NamedBaseElement(BaseElement):
    def __init__(self, name: str | None = ...) -> None: ...

class ValuedBaseElement(BaseElement):
    def __init__(self, value: str | bytes | None = ...) -> None: ...
